# -*- coding: utf-8 -*-
# This file is part of the NEAT Project suite of libraries
# Please see the LICENSE file that should have been included as part of this
# package.


from collections import Counter, defaultdict
import matplotlib.pyplot as plt
from sumy.parsers.plaintext import PlaintextParser
from sumy.nlp.tokenizers import Tokenizer
from sumy.summarizers.lex_rank import LexRankSummarizer
from sumy.summarizers.lsa import LsaSummarizer
from sumy.summarizers.luhn import LuhnSummarizer
import random
import neattext.functions as nfx
from wordcloud import WordCloud
from textblob import TextBlob



def plot_wordcloud(docx):
    mywordcloud = WordCloud().generate(docx)
    plt.imshow(mywordcloud, interpolation="bilinear")
    plt.axis("off")
    st.pyplot()


def plot_mendelhall_curve(docx):
    word_length = [len(token) for token in docx.split()]
    word_length_count = Counter(word_length)
    sorted_word_length_count = sorted(dict(word_length_count).items())
    x, y = zip(*sorted_word_length_count)
    fig = plt.figure(figsize=(20, 10))
    plt.plot(x, y)
    plt.title("Plot of Word Length Distribution")
    plt.show()
 

def get_most_common_tokens(docx, num=10):
    clean_docx = nfx.remove_stopwords(docx)
    word_freq = Counter(clean_docx.split())
    most_common_tokens = word_freq.most_common(num)
    x, y = zip(*most_common_tokens)
    return x, y


def get_stopwords_freq(docx, num=10):
    stopwords_list = nfx.extract_stopwords(docx)
    word_freq = Counter(stopwords_list)
    most_common_tokens = word_freq.most_common(num)
    x, y = zip(*most_common_tokens)
    return x, y


def get_mendelhall_curve(docx):
    word_length = [len(token) for token in docx.split()]
    word_length_count = Counter(word_length)
    sorted_word_length_count = sorted(dict(word_length_count).items())
    x, y = zip(*sorted_word_length_count)
    return x, y


def __get_wordcloud(docx):
    mywordcloud = WordCloud().generate(docx)
    return mywordcloud


def get_most_common_pos_tags(docx, num=10):
    blob = TextBlob(docx)
    word, tag = zip(*blob.tags)
    # Get most common
    tag_freq = Counter(tag).most_common(num)
    x, y = zip(*tag_freq)
    return x, y


def text_readability(text):
    import textstat 
    docx = (text)
    results = {"flesch_reading":textstat.flesch_reading_ease(docx),
    "smog_index":textstat.smog_index(docx),
    "flesch_kincaid":textstat.flesch_kincaid_grade(docx),
    "coleman_liau":textstat.coleman_liau_index(docx),
    "ari":textstat.automated_readability_index(docx),
    "dale_chall":textstat.dale_chall_readability_score(docx),
    "difficult_words":textstat.difficult_words(docx),
    "linsear_write_formula":textstat.linsear_write_formula(docx),
    "gunning_fog":textstat.gunning_fog(docx),
    # "text_standard":textstat.text_standard(docx),
    "fernandez_huerta":textstat.fernandez_huerta(docx),
    "szigriszt_pazos":textstat.szigriszt_pazos(docx),
    "gutierrez_polini":textstat.gutierrez_polini(docx),
    "crawford":textstat.crawford(docx)}
    return results


def plot_text_readability(text):
    x,y = zip(*text_readability())
    fig = plt.figure(figsize=(20, 10))
    plt.plot(x, y)
    plt.title("Plot of Text Readability")
    plt.show()



def visualize_text(docx, token_num=10, figsize=(20, 10), rotation=40):
    """Visualize Text Supplied

    Returns: several plots such as wordcloud,mendelhall_curve,pos_tags plot etc
    Params:
            docx: text document
            token_num: number of tokens to use.Default is 10
            figsize: figsize as a tuple(20,10)
            rotation: labelrotation for labels for only bar plot

    Usage::
    >>> visualize_text(docx)

    """
    # Generate Values for each metric
    xt, yt = get_most_common_tokens(docx, num=token_num)
    xs, ys = get_stopwords_freq(docx, num=token_num)
    xm, ym = get_mendelhall_curve(docx)
    xp, yp = get_most_common_pos_tags(docx)
    mywordcloud = __get_wordcloud(docx)
    xr,yr = zip(*text_readability(docx).items())
    # Vertically Stacked Subplot
    # fig, ((ax1, ax2),(ax3, ax4)) = plt.subplots(2,2,figsize=figsize) # change to 3,2
    fig, ((ax1, ax2), (ax3, ax4), (ax5, ax6)) = plt.subplots(3, 2, figsize=figsize)
    fig.suptitle("TextViz Subplots[Vertical]")
    ax1.bar(xt, yt)
    ax1.set_title("Tokens Frequency")
    ax1.tick_params(labelrotation=rotation)
    ax2.bar(xs, ys,color="orange")
    ax2.set_title("Stopwords Frequency")
    ax2.tick_params(labelrotation=rotation)
    ax3.plot(xm, ym)
    ax3.set_title("Mendalhall Curve")
    ax4.imshow(mywordcloud, interpolation="bilinear")
    ax4.set_title("Wordcloud")
    ax5.bar(xp, yp, color="green")
    ax5.set_title("PoS Tags Frequency")
    ax5.tick_params(labelrotation=rotation)
    ax6.bar(xr, yr)
    ax6.set_title("Text Readability")
    ax6.tick_params(labelrotation=rotation)


def save_visualized_text(docx, filename,token_num=10, figsize=(20, 10), rotation=40):
    """Save the Visualized Text 

    Returns: several plots such as wordcloud,mendelhall_curve,pos_tags plot etc
    Params:
            docx: text document
            token_num: number of tokens to use.Default is 10
            figsize: figsize as a tuple(20,10)
            rotation: labelrotation for labels for only bar plot

    Usage::
    >>> save_visualized_text(docx,'myplot.png')

    """
    # Generate Values for each metric
    xt, yt = get_most_common_tokens(docx, num=token_num)
    xs, ys = get_stopwords_freq(docx, num=token_num)
    xm, ym = get_mendelhall_curve(docx)
    xp, yp = get_most_common_pos_tags(docx)
    mywordcloud = __get_wordcloud(docx)
    xr,yr = zip(*text_readability(docx).items())
    # Vertically Stacked Subplot
    # fig, ((ax1, ax2),(ax3, ax4)) = plt.subplots(2,2,figsize=figsize) # change to 3,2
    fig, ((ax1, ax2), (ax3, ax4), (ax5, ax6)) = plt.subplots(3, 2, figsize=figsize)
    fig.suptitle("TextViz Subplots[Vertical]")
    ax1.bar(xt, yt)
    ax1.set_title("Tokens Frequency")
    ax1.tick_params(labelrotation=rotation)
    ax2.bar(xs, ys,color="orange")
    ax2.set_title("Stopwords Frequency")
    ax2.tick_params(labelrotation=rotation)
    ax3.plot(xm, ym)
    ax3.set_title("Mendalhall Curve")
    ax4.imshow(mywordcloud, interpolation="bilinear")
    ax4.set_title("Wordcloud")
    ax5.bar(xp, yp, color="green")
    ax5.set_title("PoS Tags Frequency")
    ax5.tick_params(labelrotation=rotation)
    ax6.bar(xr, yr)
    ax6.set_title("Text Readability")
    ax6.tick_params(labelrotation=rotation)
    fig.savefig(filename)


def summarize_text(docx, num_sentence=2):
    """Summarize Text

    Returns: a summary of text as either textrank or lexrank extractive form of summary
    """
    # For Strings
    parser = PlaintextParser.from_string(docx, Tokenizer("english"))
    # Using LexRank
    summarizer_lex = LexRankSummarizer()
    summarizer_luhn = LuhnSummarizer()
    summarizer_lsa = LsaSummarizer()

    # Summarize Docs
    summary_for_lex = summarizer_lex(parser.document, num_sentence)
    summary_for_luhn = summarizer_luhn(parser.document, num_sentence)
    summary_for_lsa = summarizer_lsa(parser.document, num_sentence)

    # Results
    summary_results = {
        "lexrank": summary_for_lex,
        "luhn": summary_for_luhn,
        "lsa": summary_for_lsa,
    }

    return summary_results


def analyze_text(docx):
    """Analyze Text lexically"""
    pass


# Source https://github.com/adashofdata/nlp-in-python-tutorial/blob/master/5-Text-Generation.ipynb
def markov_chain(text):
    """Returns a dictionary with each word as
    a key and each value as the list of words that come after the key in the text."""

    # Tokenize the text by word, though including punctuation
    words = text.split(" ")

    # Initialize a default dictionary to hold all of the words and next words
    m_dict = defaultdict(list)

    # Create a zipped list of all of the word pairs and put them in word: list of next words format
    for current_word, next_word in zip(words[0:-1], words[1:]):
        m_dict[current_word].append(next_word)

    # Convert the default dict back into a dictionary
    m_dict = dict(m_dict)
    return m_dict


def __generate_text(chain, count=15):
    """Input a dictionary in the format of key = current word, value = list of next words
    along with the number of words you would like to see in your generated sentence."""

    # Capitalize the first word
    word1 = random.choice(list(chain.keys()))
    sentence = word1.capitalize()

    # Generate the second word from the value list. Set the new word as the first word. Repeat.
    for i in range(count - 1):
        word2 = random.choice(chain[word1])
        word1 = word2
        sentence += " " + word2

    # End it with a period
    sentence += "."
    # print(sentence)
    return sentence


def generate_text(text, num_of_words=15):
    """Returns a new sentence/text From a given text using Markov Chains
    Parameters
    ----------
    text : Main Text
    num_of_words : number of words to generate
    Returns
    ----------
    Returns a new sentence of the number of words specified
    Usage
    ------
    >>> from textcaret.functions import generate_text
    >>> t1 = "your text...here"
    >>> generate_text(t1)
    >>> generate_text(t1,20)
    #Alternatively generate 4 sentences
    >>> for i in range(4):
                    print(generate_text(t1,15))
    """
    result_dict = markov_chain(text)
    final_result_sentence = __generate_text(result_dict, num_of_words)
    return final_result_sentence




# -*- coding: utf-8 -*-
# This file is part of the NEAT Project suite of libraries
# Please see the LICENSE file that should have been included as part of this
# package.

from collections import Counter
import matplotlib.pyplot as plt
from wordcloud import WordCloud
from sumy.parsers.plaintext import PlaintextParser
from sumy.nlp.tokenizers import Tokenizer
from sumy.summarizers.lex_rank import LexRankSummarizer
from sumy.summarizers.lsa import LsaSummarizer
from sumy.summarizers.luhn import LuhnSummarizer
from textblob import TextBlob
from textcaret.functions import visualize_text,save_visualized_text


class TextViz(object):
    """docstring for TextViz

    Usage::
    >>> from textcaret import TextViz
    >>> docx = " your text document "
    >>> tv = TextViz(docx)
    >>> tv.visualize()

    """

    def __init__(self, text=None):
        super(TextViz, self).__init__()
        self.text = text

    def __repr__(self):
        return "TextViz(text='{}')".format(self.text)

    def __str__(self):
        return self.text

    def plot_most_common_tokens(self, num=10):
        word_freq = Counter(self.text.split())
        most_common_tokens = word_freq.most_common(num)
        x, y = zip(*most_common_tokens)
        fig = plt.figure(figsize=(20, 10))
        plt.bar(x, y)
        plt.title("Most Common Tokens")
        plt.xticks(rotation=45)
        plt.show()

    def plot_wordcloud(self):
        mywordcloud = WordCloud().generate(self.text)
        plt.imshow(mywordcloud, interpolation="bilinear")
        plt.axis("off")
        plt.show(block=True)

    def plot_mendelhall_curve(self):
        word_length = [len(token) for token in self.text.split()]
        word_length_count = Counter(word_length)
        sorted_word_length_count = sorted(dict(word_length_count).items())
        x, y = zip(*sorted_word_length_count)
        fig = plt.figure(figsize=(20, 10))
        plt.plot(x, y)
        plt.title("Plot of Word Length Distribution")
        plt.show()

    def visualize(self):
        return visualize_text(self.text)

    def save_figure(self,filename):
        """Save Plot or Figure
        
        usage:
        >>> vis.save_figure('myplot.png')

        """
        return save_visualized_text(self.text,filename)


class TextSummarizer(object):
    """TextSummarizer: summarize a given document/text using several summarization
    algorithms such as lexrank,luhn,lsa,etc

    Returns: A Dictionary of various summary per the each extractive algorithm

    Usage::
    >>> from textcaret import TextSummarizer
    >>> s = "your text"
    >>> summarizer = TextSummarizer(s)
    >>> summarizer.summarize()
    >>> # Number of Sentence Summary to Generate
    >>> summarizer.summarize(num_sentence=3)

    """

    def __init__(self, text=None):
        super(TextSummarizer, self).__init__()
        self.text = text

    def __repr__(self):
        return "TextSummarizer(text='{}')".format(self.text)

    def __str__(self):
        return self.text

    def summarize(self, num_sentence=2):
        docx = self.text
        # For Strings
        parser = PlaintextParser.from_string(docx, Tokenizer("english"))
        # Using LexRank
        summarizer_lex = LexRankSummarizer()
        summarizer_luhn = LuhnSummarizer()
        summarizer_lsa = LsaSummarizer()

        # Summarize Docs
        summary_for_lex = summarizer_lex(parser.document, num_sentence)
        summary_for_luhn = summarizer_luhn(parser.document, num_sentence)
        summary_for_lsa = summarizer_lsa(parser.document, num_sentence)

        # Results
        summary_results = {
            "lexrank": summary_for_lex,
            "luhn": summary_for_luhn,
            "lsa": summary_for_lsa,
        }

        return summary_results


class TextSentiment(object):
    """docstring for TextSentiment"""

    def __init__(self, text=None, split_sentence=False):
        super(TextSentiment, self).__init__()
        self.text = text
        self.split_sentence = False

    def __repr__(self):
        return 'TextSentiment(text="{}",split_sentence="{}")'.format(
            self.text, self.split_sentence
        )

    def __str__(self):
        return self.text

    def sentiment(self):
        if self.split_sentence == True:
            sentence_tokens = [sent for sent in str(self.text).split(".")]
            sentiment_list = []
            for sent in sentence_tokens:
                sentiment = TextBlob(sent).sentiment
                results = (sent, sentiment.polarity)
                sentiment_list.append(results)
            sentiment_results = {"sentiment": sentiment_list}
        else:
            blob = TextBlob(self.text)
            sentiment = blob.sentiment
            sentiment_results = {"sentence": self.text, "sentiment": sentiment}
        return sentiment_results


class TextCaret(TextViz, TextSummarizer, TextSentiment):
    """TextCaret: main object for nlp task

    Returns: TextCaret Object

    Usage::
    >>> from textcaret import TextCaret
    >>> s = "your text"
    >>> docx = TextCaret(s)
    >>> docx.visual_report()
    >>> docx.summary_report()
    >>> docx.sentiment_report()
    >>> docx.general_report()

    """

    def __init__(self, text=None):
        super(TextCaret, self).__init__()
        self.text = text

    def __repr__(self):
        return "TextCaret(text='{}')".format(self.text)

    def __str__(self):
        return self.text

    def prepare(self, stopwords=False, special_char=False, punctuations=False):
        import neattext.functions as nfx

        if stopwords == True:
            self.text = nfx.remove_stopwords(self.text)
        if special_char == True:
            self.text = nfx.remove_special_characters(self.text)
        else:
            self.text = self.text

        prepared_text = self.text
        return prepared_text

    def visual_report(self):
        """Visualize The Given Text
        Generate a visual report of several plot for the given text


        """
        new_docx = TextViz(self.text)
        # Generate Plots
        return new_docx.visualize()

    def summary_report(self,num_sentence=2):
        """Summarize the Given Text
        Generate a summary report of the given text

        Usage::
        >>> docx = TextCaret("your text here")
        >>> docx.summary_report()
        >>>
        >>> docx.summary_report(num_sentence=1)

        """
        new_docx = TextSummarizer(self.text)
        # Generate Summary
        return new_docx.summarize(num_sentence)

    def sentiment_report(self):
        """Perform Sentiment Analysis on Text
        Generate a sentiment report of the given text

        Usage::
        >>> docx = TextCaret("your text here")
        >>> docx.sentiment_report()
        >>>  
        """

        new_docx = TextSentiment(self.text,split_sentence=True)
        # Generate Sentiment Report
        return new_docx.sentiment()

    def general_report(self):
        """Generate A General Report of the Analysis of the Text

        Usage:
        >>> docx = TextCaret("your text here")
        >>> docx.general_report()

        """
        import neattext as nt
        docx = nt.TextFrame(self.text)
        return docx.describe()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['textcaret', 'textcaret.functions']

package_data = \
{'': ['*']}

install_requires = \
['matplotlib>=3.4.2,<4.0.0',
 'neattext>=0.1.0,<0.2.0',
 'nltk>=3.6.2,<4.0.0',
 'spacy>=3.1.1,<4.0.0',
 'sumy>=0.8.1,<0.9.0',
 'textblob>=0.15.3,<0.16.0',
 'textstat>=0.7.1,<0.8.0',
 'wordcloud>=1.8.1,<2.0.0']

setup_kwargs = {
    'name': 'textcaret',
    'version': '0.0.1',
    'description': 'Simplified NLP Toolkit for unifying common Natural Language Processing Tasks',
    'long_description': '# textcaret\nSimplified NLP Toolkit for common NLP Tasks. Unifying common Natural Language Processing task\n\n\n#### Why TextCaret\n+ The problem: performing common NLP Task such as summarization,sentiment analysis,etc are essential however you may need to use different libraries and write different codes for performing the same set of task on different texts\n+ The goal of TextCaret is to simplify this task by providing a unified framework to \nperform these common NLP Tasks\n\n\n\n\n#### Installation\ntextcaret is available on pypi hence you can install using pip\n```bash\npip install textcaret\n```\n\n#### Benefits and Features\n+ Generate Reports for Text Analysis\n+ Text Summarization\n+ Text Visualization\n\t- wordcloud\n\t- word frequency plots\n\t- word length distribution\n\t- etc\n+ Sentiment Analysis\n+ Text Generation *\n+ etc\n\n\n#### Usage\n```python\n>>> import textcaret as tc \n>>> docx = tc.TextCaret(text=\'your text goes here\')\n>>>\n>>> docx.visual_report()\n>>> docx.summary_report()\n>>> docx.sentiment_report()\n>>> docx.general_report()\n```\n\n#### Perform Text Visualization For Insights\n+ This generates wordcloud plots,token and tags frequency plots, word length distribution and more.\n```python\n>>> from textcaret import TextViz\n>>> s = "your text"\n>>> viz = TextViz(s)\n>>> viz.visualize()\n>>> # Save Plot\n>>> viz.safe_figure(\'mynewplot.png\')\n```\n\n#### Perform TextSummarization\n+ In NLP Text Summarization is the process of shortening a set of data computationally, to create a subset (a summary) that represents the most important or relevant information within the original content.[wiki]\n+ It is the process of finding the most informative sentence in a document.\n+ TextCaret uses several extractive algorithms for generating summary\n\n```python\n>>> from textcaret import TextSummarizer\n>>> s = "your text"\n>>> summarizer = TextSummarizer(s)\n>>> summarizer.summarize()\n```\n\n#### Perform Sentiment Analysis\n+ In NLP, Sentiment Analysis is the process of identifying the emotions/sentiment or feeling in a given text either as positive,negative or neutral.\n+ It is a form of text classification\n+ TextCaret uses the famous textblob library behind the scene to generate sentiments of given text\n\n```python\n>>> from textcaret import TextSentiment\n>>> docx = TextSentiment("I love coding and teaching.John hates mangoes so bad he doesn\'t eat it")\n>>> \n>>> docx.sentiment()\n{\'sentence\': "I love coding and teaching.John hates mangoes so bad he doesn\'t eat it", \'sentiment\': Sentiment(polarity=-0.09999999999999992, subjectivity=0.6333333333333333)}\n>>> \n>>> docx.sentiment()[\'sentiment\']\nSentiment(polarity=-0.09999999999999992, subjectivity=0.6333333333333333)\n>>> \n>>> docx.sentiment()[\'sentiment\'].polarity\n-0.09999999999999992\n>>> \n>>> docx.sentiment()[\'sentiment\'].subjectivity\n0.6333333333333333\n>>> \n```\n\n####  Perform Sentiment on Splitted/Tokenized Sentences\n```python\n>>> docx.split_sentence=True\n>>> \n>>> docx.sentiment()\n{\'sentiment\': [(\'I love coding and teaching\', 0.5), ("John hates mangoes so bad he doesn\'t eat it", -0.6999999999999998)]}\n>>> \n\n```\n\n\n#### Dependencies\nTextcaret is built ontop of powerful and common NLP libraries such as below\n+ NLTK\n+ TextBlob\n+ Sumy\n+ Neattext\n+ Matplotlib\n+ Wordcloud\n+ Spacy\n\n#### .\n+ Maintainer: Jesse E.Agbe(JCharis)\n+ Jesus Saves @JCharisTech\n\n#### Contributions\n+ Notice a bug, please let us know\n+ We appreciate contributions of anykind.\n+ Happy Coding!!! :smiley:\n\n\n\n\n\n\n\n',
    'author': 'Jesse E.Agbe(JCharis)',
    'author_email': 'jcharistech@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jcharistech/textcaret',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
